@@ -1,24 +1,28 @@
-/* 
+/*
  *  IMS Open Corpus Workbench (CWB)
  *  Copyright (C) 1993-2006 by IMS, University of Stuttgart
  *  Copyright (C) 2007-     by the respective contributers (see file AUTHORS)
- * 
+ *
  *  This program is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
  *  Free Software Foundation; either version 2, or (at your option) any later
  *  version.
- * 
+ *
  *  This program is distributed in the hope that it will be useful, but
  *  WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
  *  Public License for more details (in the file "COPYING", or available via
  *  WWW at http://www.gnu.org/copyleft/gpl.html).
  */
-
-
-#include "server.h"
-#include "auth.h"
-#include "cqi.h"
+#ifdef __MINGW__
+#include <winsock2.h>
+#define socklen_t int
+#endif
+#include <signal.h>
+#include <string.h>
+#include <unistd.h>
+#include <stdio.h>
+#include <stdarg.h>
 
 #include <sys/types.h>
 #include <sys/time.h>
@@ -28,27 +32,26 @@
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <netdb.h>
-#else
-#include <winsock2.h>
-#define socklen_t int
 #endif
 
-#include <signal.h>
-#include <string.h>
-#include <unistd.h>
-#include <stdio.h>
 
 #include "../cl/cl.h"           /* this is the local cl.h header */
-#include "../cl/macros.h"
 
 #include "../cqp/options.h"
 #include "../cqp/corpmanag.h"
 #include "../cqp/parse_actions.h"
-#include "../cqp/hash.h"
-#include "../cl/lexhash.h" /* inserted by AB */
 
-#define NETBUFSIZE 512
+#include "server.h"
+#include "auth.h"
+#include "log.h"
+#include "cqi.h"
+
+/** the att hash is initially sized to 2 to the power of 14 */
 #define ATTHASHSIZE 16384
+
+void Rprintf(const char *, ...);
+
+/** Error strings are limited to this many bytes. @see cqi_error_string */
 #define GENERAL_ERROR_SIZE 1024
 
 #ifndef MSG_WAITALL
@@ -58,8 +61,6 @@
 #define MSG_WAITALL 0
 #endif
 
-void Rprintf(const char *, ...);
-
 /*
  *
  * Global variables for CQPSERVER
@@ -72,20 +73,20 @@ FILE *conn_out;                   /**< Connection out: stream for buffered outpu
 struct sockaddr_in my_addr, client_addr;
 struct hostent *remote_host;
 char *remote_address;
-cqi_byte netbuf[NETBUFSIZE];      /* do we need it at all? */
-int bytes;                        /* always used for data held in netbuf[] */
 
 
 int cqi_errno = CQI_STATUS_OK;    /**< CQi last error */
-char cqi_error_string[GENERAL_ERROR_SIZE] = "No error.";  /**< String describing the last CQi error.
-                                                           *   This can be queried by the client. */
+
+/** String describing the last CQi error. This can be queried by the client. */
+char cqi_error_string[GENERAL_ERROR_SIZE] = "No error.";
+
+
 
 /*
  *
  *  Error messages
  *
  */
-/* TODO: better names might be cqi_error_send, cqi_error_recv, cqi_error_internal */
 
 /**
  * Prints an error message to cqpserver's STDERR and then exits.
@@ -94,11 +95,10 @@ char cqi_error_string[GENERAL_ERROR_SIZE] = "No error.";  /**< String describing
  *
  * @param function  String containing the name of the function where the error was raised.
  */
-void 
+void
 cqi_send_error(char *function)
 {
-  Rprintf("ERROR CQi data send failure in function\n");
-  Rprintf("ERROR %s() <server.c>\n", function);
+  cqiserver_log(Error, "ERROR CQi data send failure in function\n\t%s() <server.c>", function);
   exit(1);
 }
 
@@ -109,11 +109,10 @@ cqi_send_error(char *function)
  *
  * @param function  String containing the name of the function where the error was raised.
  */
-void 
+void
 cqi_recv_error(char *function)
 {
-  Rprintf("ERROR CQi data recv failure in function\n");
-  Rprintf("ERROR %s() <server.c>\n", function);
+  cqiserver_log(Error, "ERROR CQi data recv failure in function\n\t%s() <server.c>\n", function);
   exit(1);
 }
 
@@ -128,9 +127,7 @@ cqi_recv_error(char *function)
 void
 cqi_internal_error(char *function, char *cause)
 {
-  Rprintf("ERROR Internal error in function\n");
-  Rprintf("ERROR %s() <server.c>\n", function);
-  Rprintf("ERROR ''%s''\n", cause);
+  cqiserver_log(Error, "ERROR Internal error in function\n\t%s() <server.c>\n\t''%s''", function, cause);
   exit(1);
 }
 
@@ -146,19 +143,22 @@ cqi_internal_error(char *function, char *cause)
  * whence it can be accessed by the client if the CQI_CTRL_LAST_GENERAL_ERROR
  * is sent to the server.
  *
- * TODO a better name would be cqi_error_general
  * @param errstring  String containing the error message.
  */
 void
 cqi_general_error(char *errstring)
 {
-  if (strlen(errstring) >= GENERAL_ERROR_SIZE) 
+  if (strlen(errstring) >= GENERAL_ERROR_SIZE)
     cqi_internal_error("cqi_general_error", "Error message too long.");
   strcpy(cqi_error_string, errstring);
   cqi_command(CQI_ERROR_GENERAL_ERROR);
 }
 
-
+/*
+ *
+ *  Connection
+ *
+ */
 
 /**
  * Wait for, and then process, an attempt by a client to initiate a connection
@@ -166,22 +166,20 @@ cqi_general_error(char *errstring)
  *
  * Note that this function may or may not fork the cqpserver process.
  *
- * If forking happens, then the child handles the connection, whereas the parent carries
- * on waiting for further connections.
+ * If forking happens, then the child handles the connection,
+ * whereas the parent carries on waiting for further connections.
  *
  * On Windows, forking never happens (since Windows doesn't support it).
  *
- * On *nix, forking happens UNLESS the global private_server is true. (Actually, if
- * private_server is true, then forking still happens, but the parent process immeidately
- * exits.)
- *
- * TODO: a better name would be server_accept_connection or somesuch...
+ * On *nix, forking happens UNLESS the global private_server is true.
+ * (Actually, if private_server is true, then forking still happens,
+ * but the parent process immeidately exits.)
  *
  * @param port  The integer identifier of the port to listen on.
- * @return      A > 0 value (actually the socket ID of the incoming connection)
+ * @return      A > 0 value ( the socket ID of the incoming connection)
  *              if all is OK; otherwise -1.
  */
-int 
+int
 accept_connection(int port)
 {
   const int on = 1;
@@ -197,12 +195,10 @@ accept_connection(int port)
   }
 #endif
 
-  if (port <= 0) {
+  if (port <= 0)
     port = CQI_PORT;
-  }
 
-  if (server_debug) 
-    Rprintf("CQi: Opening socket and binding to port %d\n", port);
+  cqiserver_debug_msg("Opening socket and binding to port %d", port);
 
 #ifdef __MINGW__
   /* startup the use of the Winsock DLL */
@@ -235,9 +231,9 @@ accept_connection(int port)
 
   my_addr.sin_family = AF_INET;
   my_addr.sin_port = htons(port);
-  if (localhost) 
+  if (localhost)
     my_addr.sin_addr.s_addr = inet_addr("127.0.0.1"); /* loopback device */
-  else 
+  else
     my_addr.sin_addr.s_addr = htonl(INADDR_ANY);      /* all network devices on local machine */
   memset(&(my_addr.sin_zero), '\0', 8);
   if (0 != bind(sockfd, (struct sockaddr *)&my_addr, sizeof(struct sockaddr))) {
@@ -245,8 +241,7 @@ accept_connection(int port)
     return -1;
   }
 
-  if (server_log)
-    Rprintf("Waiting for client on port #%d.\n", port);
+  cqiserver_log(Info, "Waiting for client on port #%d.\n", port);
   if (0 != listen(sockfd, 5)) {
     perror("ERROR listen() failed");
     return -1;
@@ -257,9 +252,9 @@ accept_connection(int port)
   if (server_quit) {
     pid_t pid = fork();
     if (pid != 0) {
-      close(sockfd);            /* parent returns to caller */
-      Rprintf("[CQPserver running in background now]\n");
-      exit(0);
+      /* parent returns to caller */
+      close(sockfd);
+      exit(cqiserver_log(Info, "[child is running in background now, parent server quits]") || cqp_error_status);
     }
   }
 #else
@@ -271,17 +266,14 @@ accept_connection(int port)
     if (private_server) {
       struct timeval tv;
       fd_set read_fd;
-      
+
       tv.tv_sec = 10;
       tv.tv_usec = 0;
       FD_ZERO(&read_fd);        /* select() on sockfd */
       FD_SET(sockfd, &read_fd);
-      
-      if ((select(sockfd+1, &read_fd, NULL, NULL, &tv) <= 0)
-          || (!FD_ISSET(sockfd, &read_fd))) {
-        Rprintf("Port #%d timed out in private server mode. Aborting.\n", port);
-        exit(1);
-      }
+
+      if (0 >= select(sockfd+1, &read_fd, NULL, NULL, &tv) || !FD_ISSET(sockfd, &read_fd))
+        exit(cqiserver_log(Error, "Port #%d timed out in private server mode. Aborting.", port) || cqp_error_status);
     }
 
     connfd = accept(sockfd, (struct sockaddr *)&client_addr, &sin_size);
@@ -289,18 +281,12 @@ accept_connection(int port)
       perror("ERROR Can't establish connection");
       return -1;
     }
-    
-    if (server_debug) 
-      Rprintf("CQi: Connection established. Looking up client's name.\n");
+
+    cqiserver_debug_msg("Connection established. Looking up client's name.");
     remote_address = inet_ntoa(client_addr.sin_addr);
     remote_host = gethostbyaddr((void *)&(client_addr.sin_addr), 4, AF_INET);
-    if (server_log) {
-      Rprintf("Connection established with %s ", remote_address);
-      if (remote_host != NULL) 
-        Rprintf("(%s)", remote_host->h_name);
-      Rprintf("\n");
-    }
-    
+    cqiserver_log(Info, "Connection established with %s (%s)", remote_address, remote_host ? remote_host->h_name : "name unknown");
+
 #ifndef __MINGW__
     /* spawn a server to handle the request */
     child_pid = fork();
@@ -308,18 +294,18 @@ accept_connection(int port)
       perror("ERROR can't fork() server");
       return -1;
     }
-    
-    if (child_pid == 0)
+
+    if (0 == child_pid)
       break;                    /* the child exits the listen() loop */
 
-    /* this is the listening 'parent', which exits immediately */
-    Rprintf("Spawned CQPserver, pid = %d.\n", (int)child_pid);
+    /* this is the listening 'parent', which exits immediately in private_server mode */
+    cqiserver_log(Info, "Spawned child CQPserver, pid = %d.", (int)child_pid);
     close(connfd);              /* this is the child's connection */
 
     if (private_server) {
-      Rprintf("Accepting no more connections (private server).\n");
       close(sockfd);
-      exit(0);                  /* SIGCHLD should be reaped by calling process */
+      exit(cqiserver_log(Info, "Accepting no more connections (private server).") || cqp_error_status);
+      /* SIGCHLD should be reaped by calling process */
     }
 #else
     /* no forking in Windows!
@@ -330,38 +316,34 @@ accept_connection(int port)
      *      http://www.gamedev.net/community/forums/topic.asp?topic_id=360290
      *      http://stackoverflow.com/questions/985281/what-is-the-closest-thing-windows-has-to-fork
      *      )
-     * We print the "private server" message automatically.
+     * We print the "private server" message unconditionally.
      */
-    Rprintf("Accepting no more connections (private server).\n");
+    cqiserver_log(Info, "Accepting no more connections (private server).\n");
     break;
 #endif
-  }/* endwhile 42 */
+  }/* endwhile true */
 
   /* this is the child serving the new CQi connection */
-  if (server_debug) 
-    Rprintf("CQi: ** new CQPserver created, initiating CQi session\n");
+  cqiserver_debug_msg("** new CQPserver created, initiating CQi session");
   close(sockfd);
 
   /* check if remote host is in validation list */
   if (!check_host(client_addr.sin_addr)) {
-    Rprintf("WARNING %s not in list, connection refused!\n", remote_address);
-    Rprintf("Exit. (pid = %d)\n", (int)getpid());
+    cqiserver_log(Info, "WARNING %s not in list, connection refused!\n", remote_address);
+    cqiserver_log(Info, "Exit. (pid = %d)\n", (int)getpid());
     close(connfd);
     exit(1);
   }
 
 #ifndef __MINGW__
   /* create a buffered stream to the outgoing connection */
-  conn_out = fdopen(connfd, "w");
-  if (conn_out == NULL) {
+  if (!(conn_out = fdopen(connfd, "w"))) {
     perror("ERROR Can't switch CQi connection to buffered output");
     close(connfd);
     return -1;
   }
 #endif
-
-  if (server_debug) 
-    Rprintf("CQi: creating attribute hash (size = %d)\n", ATTHASHSIZE);
+  cqiserver_debug_msg("creating attribute hash (size = %d)", ATTHASHSIZE);
   make_attribute_hash(ATTHASHSIZE);
 
   return connfd;
@@ -369,7 +351,8 @@ accept_connection(int port)
 
 
 
-/* 
+
+/*
  *
  *  Server -> Client communication
  *
@@ -386,22 +369,18 @@ accept_connection(int port)
  *
  * @return  Boolean: true if everything OK, otherwise false.
  */
-int 
+int
 cqi_flush(void)
 {
 #ifdef __MINGW__
   return 1;
 #else
-  if (snoop) {
-    Rprintf("CQi FLUSH\n");
-  }
+  cqiserver_snoop("FLUSH");
   if (EOF == fflush(conn_out)) {
     perror("ERROR cqi_flush()");
     return 0;
   }
-  else {
-    return 1;
-  }
+  return 1;
 #endif
 }
 
@@ -420,32 +399,29 @@ cqi_flush(void)
  * @param nosnoop  Boolean: if true, snoop functionality is overridden (to allow for non-repetition
  *                 of messages when called from a function that has already printed a message)
  */
-int 
+int
 cqi_send_byte(int n, int nosnoop)
 {
 #ifdef __MINGW__
-  unsigned char prep;
-  prep = (unsigned char) 0xff & n;
+  const char prep = (const char) 0xff & n;
+  /* prep = (unsigned char) 0xff & n; */
 #endif
 
-  if (snoop && !nosnoop) {
-    Rprintf("CQi SEND BYTE   %02X        [= %d]\n", n, n);
-  }
+  if (!nosnoop)
+    cqiserver_snoop("SEND BYTE   %02X        [= %d]", n, n);
 
   /* note that the actual sending is wrapped in an "if" whose content differs between OSes */
   if (
 #ifndef __MINGW__
-      (EOF == putc(0xff & n, conn_out))
+      EOF == putc(0xff & n, conn_out)
 #else
-      (1 != send(connfd, &prep, 1, MSG_WAITALL))
+      1 != send(connfd, &prep, 1, MSG_WAITALL)
 #endif
       )  {
     perror("ERROR cqi_send_byte()");
     return 0;
   }
-  else {
-    return 1;
-  }
+  return 1;
 }
 
 /**
@@ -457,24 +433,19 @@ cqi_send_byte(int n, int nosnoop)
  * This function should be called via one of the cqi_data_* functions
  * and not on its own.
  */
-int 
+int
 cqi_send_word(int n)
 {
-  if (snoop) {
-    Rprintf("CQi SEND WORD   %04X      [= %d]\n", n, n);
-  }
+  cqiserver_snoop("SEND WORD   %04X      [= %d]", n, n);
   if (
       /* exploit the fact that cqi_send_byte() only uses the lowest 8 bytes of its argument */
       !(cqi_send_byte(n>>8, 1)) ||
       !(cqi_send_byte(n, 1))
-      )
-    {
-      perror("ERROR cqi_send_word()");
-      return 0;
-    }
-  else {
-    return 1;
+      ) {
+    perror("ERROR cqi_send_word()");
+    return 0;
   }
+  return 1;
 }
 
 /**
@@ -487,26 +458,21 @@ cqi_send_word(int n)
  *
  * @return  Boolean: true if everything OK, otherwise false.
  */
-int 
+int
 cqi_send_int(int n)
 {
-  if (snoop) {
-    Rprintf("CQi SEND INT    %08X  [= %d]\n", n, n);
-  }
+  cqiserver_snoop("SEND INT    %08X  [= %d]", n, n);
   if (
       /* exploit the fact that cqi_send_byte() only uses the lowest 8 bytes of its argument */
       !(cqi_send_byte(n>>24, 1)) ||
       !(cqi_send_byte(n>>16, 1)) ||
       !(cqi_send_byte(n>>8, 1)) ||
       !(cqi_send_byte(n, 1))
-      )
-    {
-      perror("ERROR cqi_send_int()");
-      return 0;
-    }
-  else {
-    return 1;
+      ) {
+    perror("ERROR cqi_send_int()");
+    return 0;
   }
+  return 1;
 }
 
 
@@ -522,36 +488,32 @@ cqi_send_int(int n)
  *
  * @return  Boolean: true if everything OK, otherwise false.
  */
-int 
-cqi_send_string(char *str)
+int
+cqi_send_string(const char *str)
 {
   int len;
 
-  if (str == NULL) {
-    if (! cqi_send_word(0)) {
+  if (!str) {
+    if (!cqi_send_word(0)) {
       perror("ERROR cqi_send_string()");
       return 0;
     }
-    else {
-      return 1;
-    }
+    return 1;
   }
 
   len = strlen(str);
-  if (! cqi_send_word(len)) {
+  if (!cqi_send_word(len)) {
      perror("ERROR cqi_send_string()");
      return 0;
   }
-  if (snoop) {
-    Rprintf("CQi SEND CHAR[] '%s'\n", str);
- }
+  cqiserver_snoop("SEND CHAR[] '%s'", str);
+
   /* we can afford to mangle len and str because we're at the end of the function */
-  while (--len >= 0) {
+  while (--len >= 0)
     if (!cqi_send_byte(*str++, 1)) {
       perror("ERROR cqi_send_string()");
       return 0;
     }
-  }
 
   return 1;
 }
@@ -563,23 +525,25 @@ cqi_send_string(char *str)
  * This function should be called via one of the cqi_data_* functions
  * and not on its own.
  *
- * @param list  pointer to a block of bytes to send.
- * @param l     the number of bytes to send.
+ * @param list        pointer to a block of bytes to send.
+ * @param l           the number of bytes to send.
+ * @param as_boolean  if true, send 1 for any true byte, 0 for 0.
  *
  * @return  Boolean: true if everything OK, otherwise false.
  */
-int 
-cqi_send_byte_list(cqi_byte *list, int l)
+int
+cqi_send_byte_list(cqi_byte *list, int l, int as_boolean)
 {
   if (!cqi_send_int(l)) {
     perror("ERROR cqi_send_byte_list()");
     return 0;
   }
   while (--l >= 0) {
-    if (!cqi_send_byte(*list++, 0)) {
+    if (!cqi_send_byte(as_boolean ? (*list ? CQI_CONST_TRUE : CQI_CONST_FALSE) : *list, 0)) {
       perror("ERROR cqi_send_byte_list()");
       return 0;
     }
+    list++;
   }
   return 1;
 }
@@ -595,7 +559,7 @@ cqi_send_byte_list(cqi_byte *list, int l)
  *
  * @return  Boolean: true if everything OK, otherwise false.
  */
-int 
+int
 cqi_send_int_list(int *list, int l)
 {
   if (!cqi_send_int(l)) {
@@ -621,7 +585,7 @@ cqi_send_int_list(int *list, int l)
  * @param list  pointer to a block of pointers-to-strings; the strings will be sent.
  * @param l     the number of strings to send.
  */
-int 
+int
 cqi_send_string_list(char **list, int l)
 {
   if (!cqi_send_int(l)) {
@@ -642,56 +606,51 @@ cqi_send_string_list(char **list, int l)
 /**
  * Sends a general CQi command, without any arguments.
  */
-void 
+void
 cqi_command(int command)
 {
-  if (!cqi_send_word(command) || !cqi_flush()) {
+  if (!cqi_send_word(command) || !cqi_flush())
     cqi_send_error("cqi_command");
-  }
 }
 
 /**
  * Sends a byte of data to the client.
  */
-void 
+void
 cqi_data_byte(int n)
 {
-  if (!cqi_send_word(CQI_DATA_BYTE) || !cqi_send_byte(n, 0) || !cqi_flush()) {
+  if (!cqi_send_word(CQI_DATA_BYTE) || !cqi_send_byte(n ? CQI_CONST_TRUE : CQI_CONST_FALSE, 0) || !cqi_flush())
     cqi_send_error("cqi_data_byte");
-  }
 }
 
 /**
  * Sends a boolean to the client.
  */
-void 
+void
 cqi_data_bool(int n)
 {
-  if (!cqi_send_word(CQI_DATA_BOOL) || !cqi_send_byte(n, 0) || !cqi_flush()) {
+  if (!cqi_send_word(CQI_DATA_BOOL) || !cqi_send_byte(n ? CQI_CONST_TRUE : CQI_CONST_FALSE, 0) || !cqi_flush())
     cqi_send_error("cqi_data_bool");
-  }
 }
 
 /**
  * Sends an integer to the client.
  */
-void 
+void
 cqi_data_int(int n)
 {
-  if (!cqi_send_word(CQI_DATA_INT) || !cqi_send_int(n) || !cqi_flush()) {
+  if (!cqi_send_word(CQI_DATA_INT) || !cqi_send_int(n) || !cqi_flush())
     cqi_send_error("cqi_data_int");
-  }
 }
 
 /**
  * Sends a string to the client.
  */
-void 
-cqi_data_string(char *str)
+void
+cqi_data_string(const char *str)
 {
-  if (!cqi_send_word(CQI_DATA_STRING) || !cqi_send_string(str) || !cqi_flush()) {
+  if (!cqi_send_word(CQI_DATA_STRING) || !cqi_send_string(str) || !cqi_flush())
     cqi_send_error("cqi_data_string");
-  }
 }
 
 /**
@@ -700,12 +659,11 @@ cqi_data_string(char *str)
  * @param list  pointer to a block of bytes to send.
  * @param l     the number of bytes to send.
  */
-void 
+void
 cqi_data_byte_list(cqi_byte *list, int l)
 {
-  if (!cqi_send_word(CQI_DATA_BYTE_LIST) || !cqi_send_byte_list(list, l) || !cqi_flush()) {
+  if (!cqi_send_word(CQI_DATA_BYTE_LIST) || !cqi_send_byte_list(list, l, 0) || !cqi_flush())
     cqi_send_error("cqi_data_byte_list");
-  }
 }
 
 /**
@@ -714,12 +672,11 @@ cqi_data_byte_list(cqi_byte *list, int l)
  * @param list  pointer to a block of booleans (each occupying one byte) to send.
  * @param l     the number of bytes to send.
  */
-void 
+void
 cqi_data_bool_list(cqi_byte *list, int l)
 {
-  if (!cqi_send_word(CQI_DATA_BOOL_LIST) || !cqi_send_byte_list(list, l) || !cqi_flush()) {
+  if (!cqi_send_word(CQI_DATA_BOOL_LIST) || !cqi_send_byte_list(list, l, 1) || !cqi_flush())
     cqi_send_error("cqi_data_bool_list");
-  }
 }
 
 /**
@@ -728,12 +685,11 @@ cqi_data_bool_list(cqi_byte *list, int l)
  * @param list  pointer to a block of integers to send.
  * @param l     the number of integers to send.
  */
-void 
+void
 cqi_data_int_list(int *list, int l)
 {
-  if (!cqi_send_word(CQI_DATA_INT_LIST) || !cqi_send_int_list(list, l) || !cqi_flush()) {
+  if (!cqi_send_word(CQI_DATA_INT_LIST) || !cqi_send_int_list(list, l) || !cqi_flush())
     cqi_send_error("cqi_data_int_list");
-  }
 }
 
 /**
@@ -742,12 +698,11 @@ cqi_data_int_list(int *list, int l)
  * @param list  pointer to a block of pointers-to-strings; the strings will be sent.
  * @param l     the number of strings to send.
  */
-void 
+void
 cqi_data_string_list(char **list, int l)
 {
-  if (!cqi_send_word(CQI_DATA_STRING_LIST) || !cqi_send_string_list(list, l) || !cqi_flush()) {
+  if (!cqi_send_word(CQI_DATA_STRING_LIST) || !cqi_send_string_list(list, l) || !cqi_flush())
     cqi_send_error("cqi_data_string_list");
-  }
 }
 
 /**
@@ -756,7 +711,7 @@ cqi_data_string_list(char **list, int l)
  * @param n1  The first integer sent
  * @param n2  The second integer sent
  */
-void 
+void
 cqi_data_int_int(int n1, int n2)
 {
   if (
@@ -765,9 +720,7 @@ cqi_data_int_int(int n1, int n2)
       !cqi_send_int(n2) ||
       !cqi_flush()
       )
-    {
-      cqi_send_error("cqi_data_int_int");
-    }
+    cqi_send_error("cqi_data_int_int");
 }
 
 /**
@@ -778,7 +731,7 @@ cqi_data_int_int(int n1, int n2)
  * @param n3  The third integer sent
  * @param n4  The fourth integer sent
  */
-void 
+void
 cqi_data_int_int_int_int(int n1, int n2, int n3, int n4)
 {
   if (
@@ -789,9 +742,7 @@ cqi_data_int_int_int_int(int n1, int n2, int n3, int n4)
       !cqi_send_int(n4) ||
       !cqi_flush()
       )
-    {
       cqi_send_error("cqi_data_int_int_int_int");
-    }
 }
 
 
@@ -801,25 +752,20 @@ cqi_data_int_int_int_int(int n1, int n2, int n3, int n4)
  *
  */
 
-int 
+int
 cqi_recv_bytes(cqi_byte *buf, int bytes)
 {
-  if (bytes <= 0) {
-    return 1;
-  }
-  else {
-    if (snoop) {
-      Rprintf("CQi RECV BYTE[%d]\n", bytes);
-    }
-    if (bytes != recv(connfd, buf, bytes, MSG_WAITALL)) {
-      perror("ERROR cqi_recv_bytes()");
-      return 0;
-    }
+  if (bytes <= 0)
     return 1;
+  cqiserver_snoop("RECV BYTE[%d]", bytes);
+  if (bytes != recv(connfd, buf, bytes, MSG_WAITALL)) {
+    perror("ERROR cqi_recv_bytes()");
+    return 0;
   }
+  return 1;
 }
 
-int 
+int
 cqi_recv_byte(void)
 {
   cqi_byte b;
@@ -827,44 +773,38 @@ cqi_recv_byte(void)
     perror("ERROR cqi_recv_byte()");
     return EOF;
   }
-  if (snoop) {
-    Rprintf("CQi RECV BYTE 0x%02X\n", b);
-  }
+  cqiserver_snoop("RECV BYTE 0x%02X", b);
   return b;
 }
 
-int 
+int
 cqi_read_byte(void)
 {
   int b = cqi_recv_byte();
-  if (b == EOF) {
+  if (b == EOF)
     cqi_recv_error("cqi_read_byte");
-  }
   return b;
 }
 
-int 
+int
 cqi_read_bool(void)
 {
-  int b = cqi_recv_byte();
-  if (b == EOF) {
+  int b = cqi_recv_byte() ? CQI_CONST_TRUE : CQI_CONST_FALSE;
+  if (b == EOF)
     cqi_recv_error("cqi_read_bool");
-  }
   return b;
 }
 
-int 
+int
 cqi_read_word(void)
 {
   int n = cqi_read_byte();
   n = (n << 8) | cqi_read_byte();
-  if (snoop) {
-    Rprintf("CQi READ WORD   %04X      [= %d]\n", n, n);
-  }
+  cqiserver_snoop("READ WORD   %04X      [= %d]", n, n);
   return n;
 }
 
-int 
+int
 cqi_read_int(void)
 {
   int n = cqi_read_byte();
@@ -875,37 +815,31 @@ cqi_read_int(void)
   n = (n << 8) | cqi_read_byte();
   if (n & 0x80000000)           /* negative 32bit quantity */
     n |= minus_bits;            /* expand to full size of int type */
-  if (snoop) {
-    Rprintf("CQi READ INT    %08X  [= %d]\n", n, n);
-  }
+  cqiserver_snoop("READ INT    %08X  [= %d]", n, n);
   return n;
 }
 
 char *
 cqi_read_string(void)
 {
-  int len;
   char *s;
+  int len = cqi_read_word();
 
-  len = cqi_read_word();
   s = (char *) cl_malloc(len + 1);
   if (!cqi_recv_bytes((cqi_byte *)s, len))
     cqi_recv_error("cqi_read_string");
   s[len] = '\0';
-  if (snoop)
-    Rprintf("CQi READ CHAR[] '%s'\n", s);
+  cqiserver_snoop("READ CHAR[] '%s'", s);
   return s;
-}  
+}
 
 int
 cqi_read_command(void)
 {
   int command;
-
-  if (server_debug)
-    Rprintf("CQi: waiting for command\n");
+  cqiserver_debug_msg("waiting for command");
   command = cqi_read_byte();
-  while (command == CQI_PAD) 
+  while (command == CQI_PAD)
     command = cqi_read_byte();
   command = (command << 8) | cqi_read_byte();
   return command;
@@ -920,15 +854,13 @@ cqi_read_byte_list(cqi_byte **list)
   if (len <= 0) {
     *list = NULL;
     return 0;
-  } 
-  else {
-    *list = (cqi_byte *) cl_malloc(len);
-    for (i=0; i<len; i++)
-      (*list)[i] = cqi_read_byte();
-    if (snoop)
-      Rprintf("CQi READ BYTE[%d]\n", len);
-    return len;
   }
+
+  *list = (cqi_byte *) cl_malloc(len);
+  for (i=0; i<len; i++)
+    (*list)[i] = cqi_read_byte();
+  cqiserver_snoop("READ BYTE[%d]", len);
+  return len;
 }
 
 int
@@ -940,15 +872,13 @@ cqi_read_bool_list(cqi_byte **list)
   if (len <= 0) {
     *list = NULL;
     return 0;
-  } 
-  else {
-    *list = (cqi_byte *) cl_malloc(len);
-    for (i=0; i<len; i++)
-      (*list)[i] = cqi_read_byte();
-    if (snoop)
-      Rprintf("CQi READ BOOL[%d]\n", len);
-    return len;
   }
+
+  *list = (cqi_byte *) cl_malloc(len);
+  for (i=0; i<len; i++)
+    (*list)[i] = cqi_read_byte() ? CQI_CONST_TRUE : CQI_CONST_FALSE;
+  cqiserver_snoop("READ BOOL[%d]", len);
+  return len;
 }
 
 int
@@ -961,43 +891,36 @@ cqi_read_int_list(int **list)
     *list = NULL;
     return 0;
   }
-  else {
-    *list = (int *) cl_malloc(len * sizeof(int));
-    for (i=0; i<len; i++)
-      (*list)[i] = cqi_read_int();
-    if (snoop)
-      Rprintf("CQi READ INT[%d]\n", len);
-    return len;
-  }
+
+  *list = (int *) cl_malloc(len * sizeof(int));
+  for (i=0; i<len; i++)
+    (*list)[i] = cqi_read_int();
+  cqiserver_snoop("READ INT[%d]", len);
+  return len;
 }
 
 int
 cqi_read_string_list(char ***list)
 {
   int i, len;
-  
+
   len = cqi_read_int();
   if (len <= 0) {
     *list = NULL;
     return 0;
   }
-  else {
-    *list = (char **) cl_malloc(len * sizeof(char *));
-    for (i=0; i<len; i++)
-      (*list)[i] = cqi_read_string();
-    if (snoop)
-      Rprintf("CQi READ STRING[%d]\n", len);
-    return len;
-  }
+
+  *list = (char **) cl_malloc(len * sizeof(char *));
+  for (i=0; i<len; i++)
+    (*list)[i] = cqi_read_string();
+  cqiserver_snoop("READ STRING[%d]\n", len);
+  return len;
 }
 
 
 
 
 
-
-
-
 /*
  *
  *  Naming conventions & specifier split
@@ -1005,131 +928,149 @@ cqi_read_string_list(char ***list)
  */
 
 char cqi_id_uc_first[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ_";
-char cqi_id_uc[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ_-0123456789";
+char cqi_id_uc[]       = "ABCDEFGHIJKLMNOPQRSTUVWXYZ_-0123456789";
 char cqi_id_lc_first[] = "abcdefghijklmnopqrstuvwxyz_";
-char cqi_id_lc[] = "abcdefghijklmnopqrstuvwxyz_-0123456789";
-char cqi_id_all[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz_-0123456789";
+char cqi_id_lc[]       = "abcdefghijklmnopqrstuvwxyz_-0123456789";
+char cqi_id_all[]      = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz_-0123456789";
 
-int 
-check_corpus_name(char *name) {
-  if (
-      (strchr(cqi_id_uc_first, *name) == NULL) ||
-      (strspn(name+1, cqi_id_uc) != strlen(name+1))
-      )
-    {
-      cqi_errno = CQI_ERROR_SYNTAX_ERROR;
-      return 0;
-    }
-  else {
-    cqi_errno = CQI_STATUS_OK;
-    return 1;
-  }
-}
-       
-int 
-check_attribute_name(char *name) {
-  if (
-      (strchr(cqi_id_lc_first, *name) == NULL) ||
-      (strspn(name+1, cqi_id_lc) != strlen(name+1))
-      )
-    {
-      cqi_errno = CQI_ERROR_SYNTAX_ERROR;
-      return 0;
-    }
-  else {
-    cqi_errno = CQI_STATUS_OK;
-    return 1;
-  }
-}
-       
-int 
-check_subcorpus_name(char *name) {
-  if (
-      (strchr(cqi_id_uc_first, *name) == NULL) ||
-      (strspn(name+1, cqi_id_all) != strlen(name+1))
-      )
-    {
-      cqi_errno = CQI_ERROR_SYNTAX_ERROR;
-      return 0;
-    }
-  else {
-    cqi_errno = CQI_STATUS_OK;
-    return 1;
-  }
-}
 
-/* To avoid messing with function arguments when splitting specifiers,
-   I need the following strdupto() function, which creates a duplicate
-   of <str> just as if there were a \0 at <end> */
-char *
-strdupto(char *str, char *end)
+
+/**
+ * Makes sure that an identifier complies with the specified convention regarding its initial and subsequent characters.
+ *
+ * If the identifier  does not conform, cqi_errno is set and the function returns false.
+ *
+ * @param name       The identifier to check.
+ * @param init_cap   If true, the initial char must be uppercase or underscore.
+ *                   If false, it can be lowercase or underscore.
+ * @param rest_cap   If true, subsequent chars must be uppercase or underscore.
+ *                   If false, they must be lowercase or underscore or hyphen.
+ * @param rest_any   If true, rest_cap is overridden: any valid identifier char is accepted.
+ *                   If false, the value given as rest_cap is enforced.
+ *
+ *
+ * @return    Boolean: is the identifier in accordance with convention?
+ */
+int
+check_identifier_convention(char *name, int init_cap, int rest_cap, int rest_any)
 {
-  int len = end - str;
-  char *ret, *p;
+  char *valid_init = init_cap ? cqi_id_uc_first : cqi_id_lc_first;
+  char *valid_rest = rest_any ? cqi_id_all      : (rest_cap ? cqi_id_uc : cqi_id_lc);
 
-  ret = (char *) cl_malloc(len+1);
-  for (p = ret; len > 0; len--) 
-    *p++ = *str++;
-  *p = '\0';
-  return ret;
-}
-
-/* new strings are allocated for the output arguments */
-int 
-split_attribute_spec(char *spec, char **corpus, char **attribute) {
-  char *split = strchr(spec, '.');
- 
-  if (split == NULL) {
+  if (
+      !strchr(valid_init, *name)
+      ||
+      strspn(name+1, valid_rest) != strlen(name+1)
+      ) {
     cqi_errno = CQI_ERROR_SYNTAX_ERROR;
     return 0;
   }
-  *corpus = strdupto(spec, split);
+  cqi_errno = CQI_STATUS_OK;
+  return 1;
+}
+
+// now implemented as macro in server.h:
+
+//int
+//check_corpus_name(char *name)
+//{
+//  if (
+//      !strchr(cqi_id_uc_first, *name)
+//      ||
+//      strspn(name+1, cqi_id_uc) != strlen(name+1)
+//      ) {
+//    cqi_errno = CQI_ERROR_SYNTAX_ERROR;
+//    return 0;
+//  }
+//  cqi_errno = CQI_STATUS_OK;
+//  return 1;
+//}
+//int
+//check_attribute_name(char *name)
+//{
+//  if (
+//      !strchr(cqi_id_lc_first, *name)
+//      ||
+//      strspn(name+1, cqi_id_lc) != strlen(name+1)
+//      ) {
+//    cqi_errno = CQI_ERROR_SYNTAX_ERROR;
+//    return 0;
+//  }
+//  cqi_errno = CQI_STATUS_OK;
+//  return 1;
+//}
+
+
+//int
+//check_subcorpus_name(char *name)
+//{
+//  if (
+//      !strchr(cqi_id_uc_first, *name)
+//      ||
+//      strspn(name+1, cqi_id_all) != strlen(name+1)
+//      ) {
+//    cqi_errno = CQI_ERROR_SYNTAX_ERROR;
+//    return 0;
+//  }
+//  cqi_errno = CQI_STATUS_OK;
+//  return 1;
+//}
+
+/** new strings are allocated for the output arguments */
+int
+split_attribute_spec(char *spec, char **corpus, char **attribute)
+{
+  char *split = strchr(spec, '.');
+
+  if (!split) {
+    cqi_errno = CQI_ERROR_SYNTAX_ERROR;
+    return 0;
+  }
+  *corpus = cl_strdup_to(spec, split);
   *attribute = cl_strdup(split+1);
   if (!check_corpus_name(*corpus) || !check_attribute_name(*attribute)) {
-    free(*corpus);
-    free(*attribute);
+    cl_free(*corpus);
+    cl_free(*attribute);
     return 0;                   /* cqi_errno set by check_* function */
   }
   cqi_errno = CQI_STATUS_OK;
   return 1;
 }
 
-int 
-split_subcorpus_spec(char *spec, char **corpus, char **subcorpus) {
+int
+split_subcorpus_spec(char *spec, char **corpus, char **subcorpus)
+{
   char *split = strchr(spec, ':');
 
-  if (split == NULL) {
+  if (!split) {
     *corpus = cl_strdup(spec);
     *subcorpus = NULL;
   }
   else {
-    *corpus = strdupto(spec, split);
+    *corpus    = cl_strdup_to(spec, split);
     *subcorpus = cl_strdup(split+1);
   }
   if (
-      !check_corpus_name(*corpus) || 
-      (*subcorpus != NULL && !check_subcorpus_name(*subcorpus))
-      ) 
-    {
-      cl_free(*corpus);
-      cl_free(*subcorpus);
-      return 0;                 /* cqi_errno set by check_* function */
-    }
+      !check_corpus_name(*corpus)
+      ||
+      (NULL != *subcorpus && !check_subcorpus_name(*subcorpus))
+      ) {
+    cl_free(*corpus);
+    cl_free(*subcorpus);
+    return 0;                 /* cqi_errno set by check_* function */
+  }
   cqi_errno = CQI_STATUS_OK;
   return 1;
 }
 
 char *
-combine_subcorpus_spec(char *corpus, char *subcorpus) {
+combine_subcorpus_spec(char *corpus, char *subcorpus)
+{
   char *spec;
-
-  if (subcorpus == NULL) {
-    spec = cl_strdup(corpus);
-  }
-  else {
-    spec = (char *) cl_malloc(strlen(corpus) + strlen(subcorpus) + 2);
-    sprintf(spec, "%s:%s", corpus, subcorpus);
-  }
+  if (!subcorpus)
+    return cl_strdup(corpus);
+  spec = (char *) cl_malloc(strlen(corpus) + strlen(subcorpus) + 2);
+  sprintf(spec, "%s:%s", corpus, subcorpus);
   return spec;
 }
 
@@ -1149,8 +1090,8 @@ typedef struct att_bucket {
 /** Underlying structure for the AttHashTable object.  @see AttHashTable */
 struct att_hashtable {
   AttBucket *space;             /**< the actual array of attribute buckets. */
-  int    code;
-  int    size;
+  int code;
+  int size;
 };
 
 /** An AttHashTable object contains space for a hash table of attribute-pointers. */
@@ -1162,20 +1103,16 @@ AttHashTable AttHash = NULL;
 /**
  * This function has to be called once to initialise the global attribute hash.
  *
- * TODO better name: att_hash_make
  * @see AttHash
  */
-void 
+void
 make_attribute_hash(int size)
 {
   int bytes;
-  AttHash = (AttHashTable) cl_malloc(sizeof(struct att_hashtable));
-  
-  AttHash->size = find_prime(size);
+  AttHash = (AttHashTable)cl_malloc(sizeof(struct att_hashtable));
+  AttHash->size = cl_find_prime(size);
   bytes = sizeof(AttBucket) * AttHash->size;
-
   AttHash->space = (AttBucket *)cl_malloc(bytes);
-
   memset(AttHash->space, 0, bytes);
   AttHash->code = 0;
 }
@@ -1183,55 +1120,52 @@ make_attribute_hash(int size)
 /**
  * Frees the global AttHash object and the space that it points to.
  *
- * TODO better name: att_hash_free
  * @see AttHash
  */
-void 
+void
 free_attribute_hash(void)
 {
-  if (AttHash != NULL) {
-    if (AttHash->space != NULL) 
-      free(AttHash->space);
-    free(AttHash);
-    AttHash = NULL;
-  }
+  if (AttHash)
+    cl_free(AttHash->space);
+  cl_free(AttHash);
 }
 
 /**
  * Finds an AttBucket within the global AttHash that matches the argument string.
  */
-AttBucket *
+static AttBucket *
 att_hash_lookup(char *str)
 {
   AttBucket *p, *end;
   int i = 0;
   int offset;
 
-  if (AttHash == NULL) 
+  if (!AttHash)
     cqi_internal_error("att_hash_lookup", "AttHash not initialised.");
 
   /* points to the end of the space */
   end = AttHash->space + AttHash->size;
   /* the hash value of the string */
-  offset = hash_string(str) % AttHash->size;
+  offset = cl_hash_string(str) % AttHash->size;
   /* the primary pointer into the space */
   p = AttHash->space + offset;
 
-  for(i = (int)AttHash->size/5; i>0; p++,i--) {
-    if(p >= end) p = AttHash->space;
-    if(p->string == NULL) {     /* init new bucket */
+  for (i = (int)AttHash->size/5; i>0; p++, i--) {
+    if (p >= end)
+      p = AttHash->space;
+    if (!p->string) {
+      /* init new bucket */
       p->string = cl_strdup(str);
       p->attribute = NULL;
       p->type = ATT_NONE;
       break;
     }
-    else if(strcmp(p->string, str) == 0)
+    else if (cl_streq(p->string, str))
       break;
   }
 
-  if (i == 0) 
+  if (i == 0)
     cqi_internal_error("att_hash_lookup", "Too many collisions.");
-
   return p;
 }
 
@@ -1245,19 +1179,16 @@ cqi_lookup_attribute(char *name, int type)
     CorpusList *cl;
     Attribute *attribute;
 
-    if (server_debug) {
-      Rprintf("CQi: AttHash: attribute '%s' not found, trying to open ...\n", name);
-    }
+    cqiserver_debug_msg("AttHash: attribute '%s' not found, trying to open ...", name);
 
     if (!split_attribute_spec(name, &corpus_name, &attribute_name))
       return NULL;
     cl = findcorpus(corpus_name, SYSTEM, 0);
-    if (cl == NULL || !access_corpus(cl)) {
+    if (!cl || !access_corpus(cl)) {
       cqi_errno = CQI_CQP_ERROR_NO_SUCH_CORPUS;
       return NULL;
     }
-    attribute = cl_new_attribute(cl->corpus, attribute_name, type);
-    if (attribute == NULL) {
+    if (!(attribute = cl_new_attribute(cl->corpus, attribute_name, type))) {
       cqi_errno = CQI_CL_ERROR_NO_SUCH_ATTRIBUTE;
       return NULL;
     }
@@ -1266,20 +1197,14 @@ cqi_lookup_attribute(char *name, int type)
     cqi_errno = CQI_STATUS_OK;
     return p->attribute;
   }
-  else if (p->type != type) {
-    if (server_debug) {
-      Rprintf("CQi: AttHash: attribute '%s' found, wrong attribute type.\n", name);
-    }
+  if (p->type != type) {
+    cqiserver_debug_msg("AttHash: attribute '%s' found, wrong attribute type.", name);
     cqi_errno = CQI_CL_ERROR_WRONG_ATTRIBUTE_TYPE;
     return NULL;
   }
-  else {
-    if (server_debug) {
-      Rprintf("CQi: AttHash: attribute '%s' found in hash.\n", name);
-    }
-    cqi_errno = CQI_STATUS_OK;
-    return p->attribute;
-  }
+  cqiserver_debug_msg("AttHash: attribute '%s' found in hash.", name);
+  cqi_errno = CQI_STATUS_OK;
+  return p->attribute;
 }
 
 /**
@@ -1293,22 +1218,18 @@ cqi_lookup_attribute(char *name, int type)
  * @return      Boolean: true for all OK, otherwise false
  */
 int
-cqi_drop_attribute(char *name) {
+cqi_drop_attribute(char *name)
+{
   AttBucket *p = att_hash_lookup(name);
-
-  if ((p->attribute != NULL) && 
-      (cl_delete_attribute(p->attribute))) {
+  if (p->attribute && cl_delete_attribute(p->attribute)) {
     p->attribute = NULL;
     p->type = ATT_NONE;
     return 1;
   }
-  else {
-    return 0;
-  }
+  return 0;
 }
 
 
-
 /*
  *
  *  CQP internal function wrappers
@@ -1324,21 +1245,19 @@ cqi_find_corpus(char *name)
   CorpusList *cl;
   char *corpus, *subcorpus;
 
-  if (check_corpus_name(name)) {
+  if (check_corpus_name(name))
     cl = findcorpus(name, SYSTEM, 0);
-  }
   else {
     if (split_subcorpus_spec(name, &corpus, &subcorpus)) {
-      free(corpus);
-      free(subcorpus);
+      cl_free(corpus);
+      cl_free(subcorpus);
       cl = findcorpus(name, SUB, 0);
     }
-    else {
+    else
       cl = NULL;                        /* cqi_errno set by split_subcorpus_spec */
-    }
   }
 
-  if (cl == NULL || !access_corpus(cl)) {
+  if (!cl || !access_corpus(cl)) {
     cqi_errno = CQI_CQP_ERROR_NO_SUCH_CORPUS;
     return NULL;
   }
@@ -1349,18 +1268,13 @@ cqi_find_corpus(char *name)
 /**
  * Activates the named corpus.
  */
-int 
+int
 cqi_activate_corpus(char *name)
 {
   CorpusList *cl;
-
-  if (server_debug) 
-    Rprintf("CQi: cqi_activate_corpus('%s');\n", name);
-  cl = cqi_find_corpus(name);
-  if (cl == NULL) 
+  cqiserver_debug_msg("cqi_activate_corpus('%s');\n", name);
+  if (!(cl = cqi_find_corpus(name)))
     return 0;
-  else {
-    set_current_corpus(cl, 0);
-    return 1;
-  }
+  set_current_corpus(cl, 0);
+  return 1;
 }
