@@ -1,13 +1,13 @@
-/* 
+/*
  *  IMS Open Corpus Workbench (CWB)
  *  Copyright (C) 1993-2006 by IMS, University of Stuttgart
  *  Copyright (C) 2007-     by the respective contributers (see file AUTHORS)
- * 
+ *
  *  This program is free software; you can redistribute it and/or modify it
  *  under the terms of the GNU General Public License as published by the
  *  Free Software Foundation; either version 2, or (at your option) any later
  *  version.
- * 
+ *
  *  This program is distributed in the hope that it will be useful, but
  *  WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
@@ -15,37 +15,37 @@
  *  WWW at http://www.gnu.org/copyleft/gpl.html).
  */
 
+void Rprintf(const char *, ...);
+
 #include <math.h>
 
 #include "../cl/cl.h"
-#include "../cl/globals.h"
+#include "../cl/cwb-globals.h"
 #include "../cl/corpus.h"
 #include "../cl/attributes.h"
 #include "../cl/storage.h"
 #include "../cl/bitio.h"
 #include "../cl/compression.h"
 
-/* doesn't seem to exist outside Solaris, so we define it here */
-#define log2(x) (log(x)/log(2.0))
-/* TODO actually log2 is now a math.h function in C99 and in POSIX... */
-
 /* ---------------------------------------------------------------------- */
 
 /** Name of the program */
-char *progname = NULL;
+/* char *progname = NULL; */
 
 /** CWB id of the corpus we are working on */
-char *corpus_id = NULL;
+/* char *corpus_id = NULL; */
+
 /** Record for the corpus we are working on */
-Corpus *corpus; 
+extern Corpus *corpus;
 
 void compressrdx_usage(char *msg, int error_code);
 void compressrdx_cleanup(int error_code);
 
 /** debug level */
-int debug = 0;
+/* extern int debug = 0; */
+
 /** where debug messages are to be sent to (stderr) */
-FILE *debug_output; /* " = stderr;" init moved to main() for Gnuwin32 compatibility */
+/* FILE *debug_output; */ /* " = stderr;" init moved to main() for Gnuwin32 compatibility */
 
 /** stores current position in a bit-write-file */
 int codepos = 0;
@@ -85,9 +85,9 @@ void write_golomb_code_am(int x, int b, BFile *bf)
   /* write the binary part */
 
   nr_sc = (1 << ub) - b;
-  
+
   if (debug)
-    fprintf(debug_output, " res=%5d CL [%3d/%3d] #sc %4d "
+    Rprintf(" res=%5d CL [%3d/%3d] #sc %4d "
             "writing %5d/%d\n",
             res, lb, ub, nr_sc,
             (res < nr_sc) ? res : res + nr_sc,
@@ -99,7 +99,7 @@ void write_golomb_code_am(int x, int b, BFile *bf)
   else {
     BFwriteWord((unsigned int)(res + nr_sc), ub, bf);
     if (res + nr_sc >= (1 << ub))
-      fprintf(stderr, "Warning: can't encode %d in %d bits\n", 
+      Rprintf("Warning: can't encode %d in %d bits\n",
               res + nr_sc, ub);
   }
 
@@ -128,7 +128,7 @@ int read_golomb_code_am(int b, BFile *bf)
   } while (bit);
 
   nr_sc = (1 << ub) - b;
-  
+
   /* read binary part, bitwise */
 
   r = 0;
@@ -139,7 +139,7 @@ int read_golomb_code_am(int b, BFile *bf)
   }
 
   if (debug)
-    fprintf(debug_output, "%8d:  Read r=%5d [%3d/%3d]  #sc=%4d, ",
+    Rprintf("%8d:  Read r=%5d [%3d/%3d]  #sc=%4d, ",
             codepos, r, lb, ub, nr_sc);
 
   if (r >= nr_sc) {
@@ -150,7 +150,7 @@ int read_golomb_code_am(int b, BFile *bf)
   }
 
   if (debug)
-    fprintf(debug_output, "final r=%d\tgap=%d\n", 
+    Rprintf("final r=%d\tgap=%d\n",
             r, r+q*b);
 
   return r + q * b;
@@ -172,30 +172,29 @@ int read_golomb_code_am(int b, BFile *bf)
  *                  (if this is null, filenames will be taken from the
  *                  attribute).
  */
-void 
-compress_reversed_index(Attribute *attr, char *output_fn)
+void
+compress_reversed_index(Attribute *attr, char *output_fn, char *corpus_id, int debug)
 {
   char *s;
   char data_fname[CL_MAX_FILENAME_LENGTH];
   char index_fname[CL_MAX_FILENAME_LENGTH];
-  
+
   int nr_elements;
   int element_freq;
   int corpus_size;
   int last_pos, gap, fpos;
 
   int b;
-
   int i, k;
 
   BFile data_file;
   FILE *index_file = NULL;
 
-  PositionStream PStream;
+  ClPositionStream PStream;
   int new_pos;
 
 
-  printf("COMPRESSING INDEX of %s.%s\n", corpus_id, attr->any.name);
+  Rprintf("COMPRESSING INDEX of %s.%s\n", corpus_id, attr->any.name);
 
   /* ensure that we do NOT use the compressed index while building the
    * compressed index (yeah, a nasty thing that). That is, load the
@@ -206,12 +205,12 @@ compress_reversed_index(Attribute *attr, char *output_fn)
     Component *comp;
 
     if ((comp = ensure_component(attr, CompRevCorpus, 0)) == NULL) {
-      fprintf(stderr, "Index compression requires the REVCORP component\n");
+      Rprintf("Index compression requires the REVCORP component\n");
       compressrdx_cleanup(1);
     }
 
     if ((comp = ensure_component(attr, CompRevCorpusIdx, 0)) == NULL) {
-      fprintf(stderr, "Index compression requires the REVCIDX component\n");
+      Rprintf("Index compression requires the REVCIDX component\n");
       compressrdx_cleanup(1);
     }
   }
@@ -241,23 +240,23 @@ compress_reversed_index(Attribute *attr, char *output_fn)
     assert(s && (cl_errno == CDA_OK));
     strcpy(index_fname, s);
   }
-  
+
   if (! BFopen(data_fname, "w", &data_file)) {
-    fprintf(stderr, "ERROR: can't create file %s\n", data_fname);
+    Rprintf("ERROR: can't create file %s\n", data_fname);
     perror(data_fname);
     compressrdx_cleanup(1);
   }
-  printf("- writing compressed index to %s\n", data_fname);
-  
+  Rprintf("- writing compressed index to %s\n", data_fname);
+
   if ((index_file = fopen(index_fname, "wb")) == NULL) {
-    fprintf(stderr, "ERROR: can't create file %s\n", index_fname);
+    Rprintf("ERROR: can't create file %s\n", index_fname);
     perror(index_fname);
     compressrdx_cleanup(1);
   }
-  printf("- writing compressed index offsets to %s\n", index_fname);
+  Rprintf("- writing compressed index offsets to %s\n", index_fname);
 
   for (i = 0; i < nr_elements; i++) {
-    
+
     element_freq = cl_id2freq(attr, i);
     if ((element_freq == 0) || (cl_errno != CDA_OK)) {
       cl_error("(aborting) token frequency == 0\n");
@@ -269,37 +268,37 @@ compress_reversed_index(Attribute *attr, char *output_fn)
       cl_error("(aborting) index read error");
       compressrdx_cleanup(1);
     }
-    
+
     b = compute_ba(element_freq, corpus_size);
-    
+
     fpos = BFposition(&data_file);
     NwriteInt(fpos, index_file);
-    
+
     if (debug)
-      fprintf(debug_output, "------------------------------ ID %d (f: %d, b: %d)\n",
+      Rprintf("------------------------------ ID %d (f: %d, b: %d)\n",
               i, element_freq, b);
-    
+
     last_pos = 0;
     for (k = 0; k < element_freq; k++) {
       if (1 != cl_read_stream(PStream, &new_pos, 1)) {
         cl_error("(aborting) index read error\n");
         compressrdx_cleanup(1);
       }
-      
+
       gap = new_pos - last_pos;
       last_pos = new_pos;
-      
+
       if (debug)
-        fprintf(debug_output, "%8d:  gap=%4d, b=%4d\n", codepos, gap, b);
-      
+        Rprintf("%8d:  gap=%4d, b=%4d\n", codepos, gap, b);
+
       write_golomb_code(gap, b, &data_file);
       codepos++;
     }
-    
+
     cl_delete_stream(&PStream);
     BFflush(&data_file);
   }
-    
+
   fclose(index_file);
   BFclose(&data_file);
 
@@ -322,13 +321,13 @@ compress_reversed_index(Attribute *attr, char *output_fn)
  *                  (if this is null, filenames will be taken from the
  *                  attribute).
  */
-void 
-decompress_check_reversed_index(Attribute *attr, char *output_fn)
+void
+decompress_check_reversed_index(Attribute *attr, char *output_fn, char *corpus_id, int debug)
 {
   char *s;
   char data_fname[CL_MAX_FILENAME_LENGTH];
   char index_fname[CL_MAX_FILENAME_LENGTH];
-  
+
   int nr_elements;
   int element_freq;
   int corpus_size;
@@ -344,7 +343,7 @@ decompress_check_reversed_index(Attribute *attr, char *output_fn)
   int true_pos;
 
 
-  printf("VALIDATING %s.%s\n", corpus_id, attr->any.name);
+  Rprintf("VALIDATING %s.%s\n", corpus_id, attr->any.name);
 
   nr_elements = cl_max_id(attr);
   if ((nr_elements <= 0) || (cl_errno != CDA_OK)) {
@@ -371,20 +370,20 @@ decompress_check_reversed_index(Attribute *attr, char *output_fn)
     assert(s && (cl_errno == CDA_OK));
     strcpy(index_fname, s);
   }
-  
+
   if (! BFopen(data_fname, "r", &data_file)) {
-    fprintf(stderr, "ERROR: can't open file %s\n", data_fname);
+    Rprintf("ERROR: can't open file %s\n", data_fname);
     perror(data_fname);
     compressrdx_cleanup(1);
   }
-  printf("- reading compressed index from %s\n", data_fname);
-  
+  Rprintf("- reading compressed index from %s\n", data_fname);
+
   if ((index_file = fopen(index_fname, "r")) == NULL) {
-    fprintf(stderr, "ERROR: can't open file %s\n", index_fname);
+    Rprintf("ERROR: can't open file %s\n", index_fname);
     perror(index_fname);
     compressrdx_cleanup(1);
   }
-  printf("- reading compressed index offsets from %s\n", index_fname);
+  Rprintf("- reading compressed index offsets from %s\n", index_fname);
 
 
   for (i = 0; i < nr_elements; i++) {
@@ -404,7 +403,7 @@ decompress_check_reversed_index(Attribute *attr, char *output_fn)
     b = compute_ba(element_freq, corpus_size);
 
     if (debug)
-      fprintf(debug_output, "------------------------------ ID %d (f: %d, b: %d)\n",
+      Rprintf("------------------------------ ID %d (f: %d, b: %d)\n",
               i, element_freq, b);
 
     pos = 0;
@@ -418,13 +417,13 @@ decompress_check_reversed_index(Attribute *attr, char *output_fn)
         compressrdx_cleanup(1);
       }
       if (pos != true_pos) {
-        fprintf(stderr, "ERROR: wrong occurrence of type #%d at cpos %d (correct cpos: %d) (on attribute: %s). Aborted.\n",
+        Rprintf("ERROR: wrong occurrence of type #%d at cpos %d (correct cpos: %d) (on attribute: %s). Aborted.\n",
                 i, pos, true_pos, attr->any.name);
         compressrdx_cleanup(1);
       }
 
     }
-    
+
     cl_delete_stream(&PStream);
     BFflush(&data_file);
   }
@@ -433,48 +432,12 @@ decompress_check_reversed_index(Attribute *attr, char *output_fn)
   BFclose(&data_file);
 
   /* tell the user it's safe to delete the REVCORP and REVCIDX components now */
-  printf("!! You can delete the file <%s> now.\n",
-         component_full_name(attr, CompRevCorpus, NULL));
-  printf("!! You can delete the file <%s> now.\n",
-         component_full_name(attr, CompRevCorpusIdx, NULL));
-  
+  Rprintf("!! You can delete the file <%s> now.\n", component_full_name(attr, CompRevCorpus, NULL));
+  Rprintf("!! You can delete the file <%s> now.\n", component_full_name(attr, CompRevCorpusIdx, NULL));
+
   return;
 }
 
-/* ---------------------------------------------------------------------- */
-
-
-/**
- * Prints a usage message and exits the program.
- *
- * @param msg         A message about the error.
- * @param error_code  Value to be returned by the program when it exits.
- */
-void 
-compressrdx_usage(char *msg, int error_code)
-{
-  if (msg)
-    fprintf(stderr, "Usage error: %s\n", msg);
-  fprintf(stderr, "\n");
-  fprintf(stderr, "Usage:  %s [options] <corpus>\n\n", progname);
-  fprintf(stderr, "Compress the index of a positional attribute. Creates .crc and .crx files\n");
-  fprintf(stderr, "which replace the corresponding .corpus.rev and .corpus.rdx files. After\n");
-  fprintf(stderr, "running this tool successfully, the latter files can be deleted.\n");
-  fprintf(stderr, "\n");
-  fprintf(stderr, "Options:\n");
-  fprintf(stderr, "  -P <att>  compress attribute <att> [default: word]\n");
-  fprintf(stderr, "  -A        compress all positional attributes\n");
-  fprintf(stderr, "  -r <dir>  set registry directory\n");
-  fprintf(stderr, "  -f <file> set output file prefix (creates <file>.crc and <file>.crx)\n");
-  fprintf(stderr, "  -d        debug mode (print messages on stderr)\n");
-  fprintf(stderr, "  -D <file> debug mode (write messages to <file>)\n");
-  fprintf(stderr, "  -T        skip validation pass ('I trust you')\n");
-  fprintf(stderr, "  -h        this help page\n\n");
-  fprintf(stderr, "Part of the IMS Open Corpus Workbench v" VERSION "\n\n");
-
-  compressrdx_cleanup(error_code);
-}
-
 /**
  * Cleans up memory prior to an (error-prompted or normal) exit.
  *
@@ -486,151 +449,11 @@ compressrdx_cleanup(int error_code)
   if (corpus)
     cl_delete_corpus(corpus);
 
-  if (debug_output != stderr)
-    fclose(debug_output);
+  /* if (debug_output != stderr) */
+    /* fclose(debug_output) */
 
-  exit(error_code);
+  return;
 }
 
 
 
-/* *************** *\
- *      MAIN()     *
-\* *************** */
-
-/**
- * Main function for cwb-compress-rdx.
- *
- * @param argc   Number of command-line arguments.
- * @param argv   Command-line arguments.
- */
-int
-main(int argc, char **argv)
-{
-  char *registry_directory = NULL;
-  char *attr_name = DEFAULT_ATT_NAME;
-  Attribute *attr;
-
-  char *output_fn = NULL;
-  char *debug_fn = NULL;
-
-  extern int optind;
-  extern char *optarg;
-  int c;
-
-  int i_want_to_believe = 0;        /* skip error checks? */
-  int all_attributes = 0;
-
-  debug_output = stderr;        /* 'delayed' init (see top of file) */
-
-  /* ------------------------------------------------- PARSE ARGUMENTS */
-
-  progname = argv[0];
-
-
-  /* parse arguments */
-  while ((c = getopt(argc, argv, "+TP:r:f:dD:Ah")) != EOF) {
-
-    switch (c) {
-      /* T: skip decompression / error checking pass ("I trust you")  */
-    case 'T':
-      i_want_to_believe++;
-      break;
-
-      /* P: attribute to compress */
-    case 'P':
-      attr_name = optarg;
-      break;
-
-      /* r: registry directory */
-    case 'r': 
-      if (registry_directory == NULL) 
-        registry_directory = optarg;
-      else {
-        fprintf(stderr, "%s: -r option used twice\n", progname);
-        compressrdx_cleanup(2);
-      }
-      break;
-      
-      /* f: filename prefix for compressed data files */
-    case 'f':
-      output_fn = optarg;
-      break;
-      
-      /* d: debug mode */
-    case 'd':
-      debug++;
-      break;
-
-      /* D: debug to file */
-    case 'D':
-      debug++;
-      debug_fn = optarg;
-      break;
-
-      /* A: compress all attributes */
-    case 'A':
-      all_attributes++;
-      break;
-
-      /* h: help page */
-    case 'h':
-      compressrdx_usage(NULL, 2);
-      break;
-
-    default: 
-      compressrdx_usage("illegal option.", 2);
-      break;
-    }
-  }
-
-  if (debug_fn)  {
-    if (strcmp(debug_fn, "-") == 0)
-      debug_output = stdout;
-    else if ((debug_output = fopen(debug_fn, "w")) == NULL) {
-      fprintf(stderr, "Can't write debug output to file %s. Aborted.", debug_fn);
-      perror(debug_fn);
-      compressrdx_cleanup(1);
-    }
-  }
-
-  /* single argument: corpus id */
-  if (optind < argc) {
-    corpus_id = argv[optind++];
-  }
-  else {
-    compressrdx_usage("corpus not specified (missing argument)", 1);
-  }
-
-  if (optind < argc) {
-    compressrdx_usage("Too many arguments", 1);
-  }
-
-  if ((corpus = cl_new_corpus(registry_directory, corpus_id)) == NULL) {
-    fprintf(stderr, "Corpus %s not found in registry %s . Aborted.\n", 
-            corpus_id,
-            (registry_directory ? registry_directory : cl_standard_registry()));
-    compressrdx_cleanup(1);
-  }
-
-  if (all_attributes) {
-    for (attr = corpus->attributes; attr; attr = attr->any.next)
-      if (attr->any.type == ATT_POS) {
-        compress_reversed_index(attr, output_fn);
-        if (! i_want_to_believe)
-          decompress_check_reversed_index(attr, output_fn);
-      }
-  }
-  else {
-    if ((attr = cl_new_attribute_oldstyle(corpus, attr_name, ATT_POS, NULL)) == NULL) {
-      fprintf(stderr, "Attribute %s.%s doesn't exist. Aborted.\n", corpus_id, attr_name);
-      compressrdx_cleanup(1);
-    }
-    compress_reversed_index(attr, output_fn);
-    if (! i_want_to_believe) 
-      decompress_check_reversed_index(attr, output_fn);
-  }
-  
-  compressrdx_cleanup(0);
-  return 0;                        /* never reached; to keep gcc from complaining */
-}
